@@ -33,8 +33,10 @@ void cl_clara(int *n,  /* = number of objects */
 	      int *jtmd,	/* [j]= {-1,1};	 -1: x[,j] has NA; 1: no NAs in x[,j] */
 	      int *diss_kind,/*= {1,2};  1 : euclidean;  2 : manhattan*/
 	      int/*logical*/ *rng_R,/*= {0,1};  0 : use clara's internal weak RNG;
-				     *	     1 : use R's RNG (and seed) */
-	      int *nrepr,
+				     *	        1 : use R's RNG (and seed) */
+	      int/*logical*/ *pam_like,/* if (1), we do "swap()" as in pam();*/
+	      // otherwise use the code as it was in clara() "forever" upto 2011-04
+ 	      int *nrepr, /* logical (0/1): 1 = "is representative object"  */
 	      int *nsel,
 	      int *nbest,/* x[nbest[j],] : the j-th obs in the final sample */
 	      int *nr, int *nrx,/* prov. and final "medoids" aka representatives */
@@ -63,7 +65,7 @@ void cl_clara(int *n,  /* = number of objects */
     Rboolean nafs, kall, full_sample, lrg_sam, dyst_toomany_NA,
 	has_NA = *mdata;
     int j, jk, jkk, js, jsm, jran, l, n_sam;
-    int nsm, ntt, nad, nadv, rand_k, kans, nrun, n_dys, nsamb, nunfs;
+    int nsm, ntt, rand_k, nrun, n_dys, nsamb, nunfs;
     double rnn, sky, zb, s, sx = -1., zba = -1.;/* Wall */
 
     *jstop = 0;
@@ -91,8 +93,8 @@ void cl_clara(int *n,  /* = number of objects */
 #define NEW_rand_k_trace_print(_nr_)					\
 	rand_k= 1+ (int)(rnn* ((*rng_R)? unif_rand(): randm(&nrun)));	\
 	if (rand_k > *n) {/* should never happen */			\
-	    REprintf("** C clara(): random k=%d > n **\n", rand_k);	\
-	    rand_k = *n;							\
+	    warning(_("C level clara(): random k=%d > n **\n"), rand_k); \
+	    rand_k = *n;						\
 	}								\
 	if(*trace_lev >= 4) {						\
 	    Rprintf("... {" #_nr_ "}");					\
@@ -101,7 +103,6 @@ void cl_clara(int *n,  /* = number of objects */
 	    Rprintf(" -> k{ran}=%d\n", rand_k);				\
 	}
 
-
 /* __LOOP__ :  random subsamples are drawn and partitioned into kk clusters */
 
     kall = FALSE; /* kall becomes TRUE iff we've found a "valid sample",
@@ -124,7 +125,7 @@ void cl_clara(int *n,  /* = number of objects */
 		    nsm = nsel[jk];
 		    jsm = jk;
 		    for (jkk = jk + 1; jkk < *kk; ++jkk) {
-			if (nsel[jkk] < nsm) {
+			if (nsm > nsel[jkk]) {
 			    nsm = nsel[jkk];
 			    jsm = jkk;
 			}
@@ -132,7 +133,6 @@ void cl_clara(int *n,  /* = number of objects */
 		    nsel[jsm] = nsel[jk]; nsel[jk] = nsm;
 		}
 		ntt = *kk;
-
 	    }
 	    else { /* no valid sample  _OR_  lrg_sam */
 		if(*trace_lev >= 2) Rprintf(" finding 1st... new k{ran}:\n");
@@ -179,19 +179,18 @@ void cl_clara(int *n,  /* = number of objects */
 		    }
 		}
 		/* insert rand_k into nsel[1:ntt] or after  and increase ntt : */
-		for (kans = 0; kans < ntt; ++kans)
-		    if (nsel[kans] >= rand_k) {
-			if (nsel[kans] == rand_k)
+		for (int ka = 0; ka < ntt; ++ka)
+		    if (nsel[ka] >= rand_k) {
+			if (nsel[ka] == rand_k)
 			    goto L210;
-			else {
-			    for (nad = kans; nad <= ntt; ++nad) {
-				nadv = ntt - nad + kans;
-				nsel[nadv] = nsel[nadv-1];
-			    }
-			    nsel[kans] = rand_k;
+			else {// nsel[ka] > rand_k :
+			    for (int na = ntt; na > ka; --na)
+				nsel[na] = nsel[na-1];
+			    nsel[ka] = rand_k;
 			    /* continue _outer_ loop */ goto L290;
 			}
 		    }
+		// else: rand_k > nsel[ka]  for all ka = 0:(ntt-1) :
 		nsel[ntt] = rand_k;
 
 	    L290:
@@ -239,14 +238,16 @@ void cl_clara(int *n,  /* = number of objects */
 	if(*trace_lev >= 2)
 	    Rprintf(". clara(): s:= max dys[1..%d] = %g;", l-1,s);
 
-	bswap2(*kk, *nsam, nrepr, dys, &sky, s,
+	bswap2(*kk, *nsam, s, dys, *pam_like, *trace_lev,
+	       /* --> */ &sky, nrepr,
 	       /* dysma */tmp1, /*dysmb*/tmp2,
 	       /* beter[], only used here */&tmp[nsamb]);
 
-	if(*trace_lev >= 2) Rprintf(" bs2");
+	if(*trace_lev >= 2)
+	    Rprintf("end{bswap}: sky = %g\n", sky);
 
 	selec(*kk, *n, *jpp, *diss_kind, &zb, *nsam, has_NA, jtmd, valmd,
-	      nrepr, nsel, dys, x, nr, &nafs, ttd, radus, ratt,
+	      *trace_lev, nrepr, nsel, dys, x, nr, &nafs, ttd, radus, ratt,
 	      ntmp1, ntmp2, ntmp3, ntmp4, ntmp5, ntmp6, tmp1, tmp2);
 
 	if (nafs) { /* couldn't assign some observation (to a cluster)
@@ -302,8 +303,9 @@ void cl_clara(int *n,  /* = number of objects */
     dysta2(*nsam, *jpp, nbest, x, *n, dys, *diss_kind, jtmd, valmd,
 	   has_NA, &dyst_toomany_NA);
     if(dyst_toomany_NA) {
-	REprintf(" *** SHOULD NOT HAPPEN: clara() -> dysta2(nbest) gave toomany_NA\n");
-	return;
+	error(_(
+	  "clara()'s C level dysta2(nsam=%d, p=%d, nbest=%d, n=%d) gave 'toomany_NA'"),
+	      *nsam, *jpp, nbest, *n );
     }
     resul(*kk, *n, *jpp, *diss_kind, has_NA, jtmd, valmd, x, nrx, mtt);
 
@@ -331,23 +333,21 @@ void dysta2(int nsam, int jpp, int *nsel,
 {
 /* Compute Dissimilarities for the selected sub-sample	---> dys[,] */
 
-    /* Local variables */
-    int j, k, kj, l, lj, ksel, lsel, nlk, npres;
-    double clk, d1, pp = (double) jpp;
-
-    nlk = 0;
+    int nlk = 0;
     dys[0] = 0.;/* very first index; *is* used because ind_2(i,i) |-> 0 ! */
-    for (l = 1; l < nsam; ++l) {
-	lsel = nsel[l];
+    for (int l = 1; l < nsam; ++l) {
+	int lsel = nsel[l];
 	if(lsel <= 0 || lsel > n)
-	    REprintf(" ** dysta2(): nsel[l= %d] = %d is OUT\n", l, lsel);
-	for (k = 0; k < l; ++k) { /* compute d(nsel[l], nsel[k]) {if possible}*/
-	    ksel = nsel[k];
+	    error(_("C level dysta2(): nsel[%s= %d] = %d is outside 0..n, n=%d"), 
+		  "l", l, lsel, n);
+	for (int k = 0; k < l; ++k) { /* compute d(nsel[l], nsel[k]) {if possible}*/
+	    int ksel = nsel[k];
 	    if(ksel <= 0 || ksel > n)
-		REprintf(" ** dysta2(): nsel[k= %d] = %d is OUT\n", k, ksel);
-	    clk = 0.;
+		error(_("C level dysta2(): nsel[%s= %d] = %d is outside 0..n, n=%d"), 
+		      "k", k, ksel, n);
 	    ++nlk;
-	    npres = 0;
+	    int npres = 0, j, lj, kj;
+	    double clk = 0.;
 	    for (j = 0, lj = lsel-1, kj = ksel-1; j < jpp;
 		 ++j, lj += n, kj += n) {
 		if (has_NA && jtmd[j] < 0) { /* x[,j] has some Missing (NA) */
@@ -367,7 +367,7 @@ void dysta2(int nsam, int jpp, int *nsel,
 		*toomany_NA = TRUE;
 		dys[nlk] = -1.;
 	    } else {
-		d1 = clk * (pp / (double) npres);
+		double d1 = clk * (jpp / (double) npres);
 		dys[nlk] = (diss_kind == 1) ? sqrt(d1) : d1 ;
 	    }
 	} /* for( k ) */
@@ -388,12 +388,15 @@ double randm(int *nrun)
 } /* randm() */
 
 /* bswap2() : called once [per random sample] from clara() : */
-void bswap2(int kk, int nsam, int *nrepr,
-	    double *dys, double *sky, double s,
+void bswap2(int kk, int n, /* == nsam == 'sampsize', here in clara */
+	    double s, const double dys[],
+	    Rboolean pam_like, int trace_lev,
+	    // result:
+	    double *sky, int *nrepr,
 	    double *dysma, double *dysmb, double *beter)
 {
-    int j, i, k, ij, kj, kbest = -1, nbest = -1;/* init for -Wall */
-    double small, asky, dzsky, dz;
+    int i, j, ij, k,h, hbest = -1, nbest = -1;/* init for -Wall */
+    double dzsky;
 
     /* Parameter adjustments */
     --nrepr;
@@ -401,24 +404,30 @@ void bswap2(int kk, int nsam, int *nrepr,
 
     --dysma;	--dysmb;
 
+    if(trace_lev >= 2) {
+	if(trace_lev == 2)
+	    Rprintf("\n bswap2():");
+	else
+	    Rprintf("\nclara()'s bswap2(*, s=%g): ", s);
+    }
+
     s = s * 1.1 + 1.;/* value larger than all dissimilarities */
 
 /* ====== first algorithm: BUILD. ====== */
 
-    for (i = 1; i <= nsam; ++i) {
+    for (i = 1; i <= n; ++i) {
 	nrepr[i] = 0;
 	dysma[i] = s;
     }
 
     for(k = 0; k < kk; k++) {
 	int nmax = -1; /* -Wall */
-	double ammax, cmd;
-	ammax = 0.;
-	for (i = 1; i <= nsam; ++i) {
+	double ammax = 0.;
+	for (i = 1; i <= n; ++i) {
 	    if (nrepr[i] == 0) {
 		beter[i] = 0.;
-		for (j = 1; j <= nsam; ++j) {
-		    cmd = dysma[j] - dys[ ind_2(i, j)];
+		for (j = 1; j <= n; ++j) {
+		    double cmd = dysma[j] - dys[ ind_2(i, j)];
 		    if (cmd > 0.)
 			beter[i] += cmd;
 		}
@@ -431,36 +440,58 @@ void bswap2(int kk, int nsam, int *nrepr,
 	}
 
 	nrepr[nmax] = 1;/* = .true. : found new representative */
+	if(trace_lev >= 2) {
+	    if(trace_lev == 2)
+		Rprintf(" %d", nmax);
+	    else
+		Rprintf("    new repr. %d\n", nmax);
+	}
 
 	/* update dysma[] : dysma[j] = D(j, nearest_representative) */
-	for (j = 1; j <= nsam; ++j) {
+	for (j = 1; j <= n; ++j) {
 	    ij = ind_2(nmax, j);
 	    if (dysma[j] > dys[ij])
 		dysma[j] = dys[ij];
 	}
     }
+    // output of the above loop:  nrepr[], dysma[], ...
 
     *sky = 0.;
-    for (j = 1; j <= nsam; ++j)
+    for (j = 1; j <= n; ++j)
 	*sky += dysma[j];
 
+    if(trace_lev >= 2) /* >= 2 (?) */ {
+	Rprintf("  after build: medoids are");
+	for (i = 1; i <= n; ++i)
+	    if(nrepr[i] == 1) Rprintf(" %2d", i);
+	if(trace_lev >= 3) {
+	    Rprintf("\n  and min.dist dysma[1:n] are\n");
+	    for (i = 1; i <= n; ++i) {
+		Rprintf(" %6.3g", dysma[i]);
+		if(i % 10 == 0) Rprintf("\n");
+	    }
+	    if(n % 10 != 0) Rprintf("\n");
+	} else Rprintf("\n");
+	Rprintf(" --> sky = sum_j D_j= %g\n", *sky);
+    }
+
     if (kk == 1)
 	return;
 
-    asky = *sky / ((double) nsam);
+// asky = *sky / ((double) n);
 
 /* ====== second algorithm: SWAP. ====== */
 
 /* Big LOOP : */
 L60:
 
-    for (j = 1; j <= nsam; ++j) {
+    for (j = 1; j <= n; ++j) {
 	/*  dysma[j] := D_j  d(j, <closest medi>)  [KR p.102, 104]
 	 *  dysmb[j] := E_j  d(j, <2-nd cl.medi>)  [p.103] */
 	dysma[j] = s;
 	dysmb[j] = s;
-	for (i = 1; i <= nsam; ++i) {
-	    if (nrepr[i] != 0) {
+	for (i = 1; i <= n; ++i) {
+	    if (nrepr[i]) {
 		ij = ind_2(i, j);
 		if (dysma[j] > dys[ij]) {
 		    dysmb[j] = dysma[j];
@@ -472,28 +503,29 @@ L60:
 	}
     }
 
-    dzsky = 1.;
-    for (k = 1; k <= nsam; ++k) if (nrepr[k] == 0) {
-	for (i = 1; i <= nsam; ++i) if (nrepr[i] != 0) {
-	    dz = 0.;
+    dzsky = 1.; /* 1 is arbitrary > 0; only dzsky < 0 matters in the end */
+    for (h = 1; h <= n; ++h) if (!nrepr[h]) {
+        for (i = 1; i <= n; ++i) if (nrepr[i]) {
+	    double dz = 0.;
 	    /* dz := T_{ih} := sum_j C_{jih}  [p.104] : */
-	    for (j = 1; j <= nsam; ++j) {
-		ij = ind_2(i, j);
-		kj = ind_2(k, j);
+	    for (j = 1; j <= n; ++j) {
+		int ij = ind_2(i, j),
+		    hj = ind_2(h, j);
 		if (dys[ij] == dysma[j]) {
-/* FIXME: The following correction *does* change results ....
- *should we do it? -- not yet*/
-		    /* correction below {from pam()}: was .. > dys[ij] ? ..*/
-		    /* small = dysmb[j] > dys[kj] ? dys[kj] : dysmb[j]; */
-		    small = dysmb[j] > dys[ij] ? dys[kj] : dysmb[j];
+		    double small;
+		    if(pam_like)
+			small = dysmb[j] > dys[hj] ? dys[hj] : dysmb[j];
+		    else // old clara code which differs from pam()'s
+			// and seems a bit illogical:
+			small = dysmb[j] > dys[ij] ? dys[hj] : dysmb[j];
 		    dz += (- dysma[j] + small);
 		}
-		else if (dys[kj] < dysma[j])
-		    dz += (- dysma[j] + dys[kj]);
+		else if (dys[hj] < dysma[j])
+		    dz += (- dysma[j] + dys[hj]);
 	    }
 	    if (dzsky > dz) {
-		dzsky = dz;
-		kbest = k;
+		dzsky = dz; // dzsky := min_{i,h} T_{i,h}
+		hbest = h;
 		nbest = i;
 	    }
 	}
@@ -503,17 +535,24 @@ L60:
     R_CheckUserInterrupt();
 
     if (dzsky < 0.) { /* found an improving swap */
-	nrepr[kbest] = 1;
+	if(trace_lev >= 3)
+	    Rprintf( "   swp new %d <-> %d old; decreasing diss. by %g\n",
+		     hbest, nbest, dzsky);
+	nrepr[hbest] = 1;
 	nrepr[nbest] = 0;
 	*sky += dzsky;
 	goto L60;
     }
+    if(trace_lev >= 2 && hbest != -1) // in my examples  hbest == -1 and it does not print:
+	Rprintf( "  Last swap: new %d <-> %d old; decreasing diss. by %g\n",
+		 hbest, nbest, dzsky);
 
 } /* End of bswap2() -------------------------------------------------- */
 
 /* selec() : called once [per random sample] from clara() */
 void selec(int kk, int n, int jpp, int diss_kind,
 	   double *zb, int nsam, Rboolean has_NA, int *jtmd, double *valmd,
+	   int trace_lev,
 	   int *nrepr, int *nsel, double *dys, double *x, int *nr,
 	   Rboolean *nafs, /* := TRUE if a distance cannot be calculated */
 	   double *ttd, double *radus, double *ratt,
@@ -525,7 +564,7 @@ void selec(int kk, int n, int jpp, int diss_kind,
     int j, jk, jj, jp, jnew, ka, kb, jkabc = -1/* -Wall */;
     int newf, nrjk,  npab, nstrt, na, nb, npa, npb, njk, nobs;
 
-    double dsum, pp = (double) (jpp), tra, rns, dnull = -9./* -Wall */;
+    double pp = (double) (jpp), tra;
 
 /* Parameter adjustments */
     --nsel;    --nrepr;
@@ -560,6 +599,7 @@ void selec(int kk, int n, int jpp, int diss_kind,
     newf = 0;
 
     for(jj = 1; jj <= n; jj++) {
+	double dsum, dnull = -9./* -Wall */;
 	if (!has_NA) {
 	    for (jk = 1; jk <= kk; ++jk) {
 		dsum = 0.;
@@ -581,8 +621,7 @@ void selec(int kk, int n, int jpp, int diss_kind,
 	    }
 	}
 	else { /* _has_ missing data */
-	    Rboolean pres;
-	    pres = FALSE;
+	    Rboolean pres = FALSE;
 	    for (jk = 1; jk <= kk; ++jk) {
 		dsum = 0.;
 		nrjk = nr[jk];
@@ -664,7 +703,7 @@ void selec(int kk, int n, int jpp, int diss_kind,
 	radus[jk] = rdnew[jk];
     }
     for (j = 1; j <= kk; ++j) {
-	rns = (double) ns[j];
+	double rns = (double) ns[j];
 	ttd[j] /= rns;
     }
 
